#!/bin/sh

. /etc/utils/shell-utils.sh
DEBUG=0

RALINK_CFG_FILE=/ram/wifi/configs/RT2870STA.dat
LOCK_FILE=/ram/lock/wifi-init.lock
LOCK_ISACTIVE_FILE=/ram/lock/wifi-isactive.lock
CURR_DRIVER_FILE=/ram/wifi/curr_driver
LOCK_OWNED=0
FW_ENV_RESULT=
WIFI_STARTUP_SCRIPT=/etc/init.d/wifi.sh

WIFI_DRIVER_MODULE_FILENAME="UNKNOWN"
WIFI_DRIVER_MODULE_NAME="UNKNOWN"

select_driver_name()
{
    if [ "mt7601" == "${wifi_driver}" ]; then
	WIFI_DRIVER_MODULE_FILENAME="mt7601Usta.ko"
	WIFI_DRIVER_MODULE_NAME="mt7601Usta"
	return;
    fi

    if [ "mt7610" == "${wifi_driver}" ]; then
	WIFI_DRIVER_MODULE_FILENAME="mt7650u_sta.ko"
	WIFI_DRIVER_MODULE_NAME="mt7650u_sta"
	return;
    fi

    if [ "8188eu" == "${wifi_driver}" ]; then
	WIFI_DRIVER_MODULE_FILENAME="8188eu.ko"
	WIFI_DRIVER_MODULE_NAME="8188eu"
	return;
    fi

    WIFI_DRIVER_MODULE_FILENAME="rt5370sta.ko"
    WIFI_DRIVER_MODULE_NAME="rt5572sta"
}

ntp_static_start()
{
    log "add ntp servers"

    # update NTP servers using DHCP info
    if [ ! -f $NTPCONF ]; then
      cat $NTPCONFDEF > $NTPCONF
      NTP_IP=`fw_printenv ntpurl 2>/dev/null`
      NTP_IP=${NTP_IP#ntpurl=}
      if [ "$NTP_IP" != "" ]; then
        echo server $NTP_IP "#manual" >> $NTPCONF
      fi
    fi
    /etc/init.d/ntp-server restart
}

set_igmp_version()
{
    igmp_conf=`fw_printenv igmp_conf 2>/dev/null`
    igmp_conf=${igmp_conf#igmp_conf=}
    if [ "$igmp_conf" == "V3" ]; then
      echo "3" > /proc/sys/net/ipv4/conf/$1/force_igmp_version
    else
      echo "2" > /proc/sys/net/ipv4/conf/$1/force_igmp_version
    fi
}


_check_lock_and_delete()
{
  if [ $LOCK_OWNED -gt 0 ]; then
    #log "Lock owned. Deleting lock..."
    rm -f $LOCK_FILE
  fi
}

_exit()
{
  _check_lock_and_delete

  exit 0;
}
_error()
{
  _check_lock_and_delete

  exit 1;
}

get_nvram_env()
{
    val=`fw_printenv $1 2>/dev/null`
    FW_ENV_RESULT=${val#*=}
    #FW_ENV_RESULT=`echo -ne $val`
}

read_nvram_env()
{
eval tmpVar=\$$1
#echo $tmpVar
  if [ "$tmpVar" != "" ]; then
    #echo "already set"
    return;
fi
  get_nvram_env $1
  export $1="$FW_ENV_RESULT"
}

_execute()
{
  if [ $DEBUG == "1" ]; then
    log "$1"
  fi

  eval $1
}

_make_ralink_init()
{
      if [ "$wifi_int_ip" == "0.0.0.0" ] || [ "$wifi_int_ip" == "" ]; then
        _execute "/etc/init.d/udhcpcd start wlan0 &"
      else
        #log "Configuting static interface settings"
        _execute "ifconfig wlan0 $wifi_int_ip netmask $wifi_int_mask"
        _execute "sleep 1"
        _execute "route add default gw $wifi_int_gw dev wlan0 metric 1"
	_execute "route add -net 224.0.0.0 netmask 240.0.0.0  dev wlan0 metric 1"
        _execute "echo -e \"nameserver \"$wifi_int_dns\" #wlan0\" >> /etc/resolv.conf"
        set_igmp_version "wlan0"
        ntp_static_start
        if [ ! -f /ram/dhcp_ready ] ; then
            echo "$wifi_int_ip" > /ram/dhcp_ready
        fi
      fi
}

_make_ralink_deinit()
{
  log "Stopping DHCP client..."

  _execute "/etc/init.d/ifplugd stop"
  _execute "/etc/init.d/udhcpcd stop wlan0 &"

  _execute "sleep 1"

  _execute "/etc/init.d/ifplugd start"

  sleep 1

  rm -f $LOCK_ISACTIVE_FILE
}


_make_ralink_cfg()
{
    log "_make_ralink_cfg *"
}

_check_variables()
{
  if [ -z "$wifi_ssid" ]; then
    wifi_ssid="nonexistentssid"
  fi

  if [ -z "$wifi_enc" ]; then
    wifi_enc="tkip"
  fi

  if [ -z "$wifi_auth" ]; then
    wifi_auth="wpa2psk"
  fi
}

stop_p()
{
        if [ ! -f $LOCK_ISACTIVE_FILE ]; then
            _exit
        fi

	wifi_driver=`cat ${LOCK_ISACTIVE_FILE}`

      log "Stopping driver(driver=$wifi_driver)"

	select_driver_name

      #wifi_driver=`cat $LOCK_ISACTIVE_FILE`
      #log "wifi_driver=$wifi_driver"

      read_nvram_env "wifi_int_ip"

      _make_ralink_deinit

      log "DONE."
}

start_p()
{
	log "Starting wifi..."

	if [ "" == "$wifi_driver" ]; then
	    log "UNDEFINED driver!!!"
	    wifi_driver=`cat ${CURR_DRIVER_FILE}`
	else 
	    echo -n $wifi_driver > $CURR_DRIVER_FILE
	fi

	#select_driver_name

      log "Starting wifi (driver=$wifi_driver)"

	read_nvram_env "wifi_ssid"
	read_nvram_env "wifi_psk"
	read_nvram_env "wifi_enc"
	read_nvram_env "wifi_auth"
	read_nvram_env "wifi_int_ip"
	read_nvram_env "wifi_int_mask"
	read_nvram_env "wifi_int_gw"
	read_nvram_env "wifi_int_dns"
	read_nvram_env "wifi_wep_def_key"
	read_nvram_env "wifi_wep_key1"
	read_nvram_env "wifi_wep_key2"
	read_nvram_env "wifi_wep_key3"
	read_nvram_env "wifi_wep_key4"
	
	log "ssid=${wifi_ssid}"
        _check_variables

        sleep 1

        log "Ralink driver section"
        echo -n $wifi_driver > $LOCK_ISACTIVE_FILE
        log "Making configuration..."
        #_make_ralink_cfg
        log "Initializing..."
        _make_ralink_init

        log "DONE."
}

# EXECUTION POINT

log "Wifi driver = $wifi_driver"

if [ -f $LOCK_FILE ]; then
  echo "Init is already running. Exiting..." >&2
  _error
fi

# make lock
echo -n > $LOCK_FILE
LOCK_OWNED=1

case $1 in
    conn_status)
          tmp_status=`iwpriv wlan0 connStatus | awk '{ if (NR!=1) exit; line1=index($0,":"); if (line10=0) {print "Unknown"; exit}; line2=index($0,"("); if (line2==0) {len=length($0)-line1} else {len=line2-line1-1}; text=substr($0, line1+1, len); print text }'`
          if [ "$tmp_status" == "Connected" ]; then
            echo "Connected"
            else
              echo "Disconnected"
          fi
          _exit
    ;;
    get_ip)
      log "Fetching IP address info for wireless interface"

      ifconfig wlan0 2>/dev/null | grep 'inet addr' | awk '{ printf substr($2, 6) }'

      log "DONE."
      ;;
    status)
      if [ -f $LOCK_ISACTIVE_FILE ]; then
        echo "running"
      else
        echo "stopped"
      fi
    ;;
    stop)
	stop_p > /dev/null
      ;;
    restart)
	log "Restarting wifi..."

	stop_p > /dev/null
	sleep 3
	start_p > /dev/null

        log "DONE."
	;;
    start)
	start_p > /dev/null
    ;;
    *)
      echo "Wireless init script v1.0"
      echo "usage: [start|stop|status|conn_status|get_ip]"
      ;;
esac

_exit
